from transformers import AutoTokenizer, AutoModel
import torch
import numpy as np
from torch.utils.data import DataLoader
import pandas as pd
from tqdm import tqdm
from loguru import logger
import random
import argparse
import json
import pandas
from typing import List


def set_seed(seed=128):
    random.seed(seed)
    torch.manual_seed(seed)
    np.random.seed(seed)
    torch.cuda.manual_seed_all(seed)


class MyTorchDataset(torch.utils.data.Dataset):
    # def __init__(self, input_file, tokenizer, max_seq_length):
    #     self.tokenizer = tokenizer
    #     self.max_seq_length = max_seq_length
    #     logger.info('Loading data: {}'.format(input_file))
    #     with open(input_file, 'r', encoding='utf8') as f:
    #         data_list = f.readlines()
    #     logger.info("There are {} data in dataset".format(len(data_list)))
    #     # 测试,这里仅仅取 top 100
    #     self.raw_data_list = data_list[:]
    #
    #     # 这个数据在初始化的时候全部 tokenizer 好,如果在训练的时候每次选择一条,会拖慢整体获取 embeding 的速度
    #     self.query_list = []
    #     self.index_list = []
    #     for line in self.raw_data_list:
    #         json_line = json.loads(line)
    #         print(json_line["data"][0])
    #         # 这个仅仅得到 query 的 embeding, content[0] 是 query
    #         query = json_line["data"][0]['content'].strip()
    #         # 这个是原始数据里，单轮对话对应的 data_index
    #         data_index = json_line["uid"]
    #         self.query_list.append(query)
    #         self.index_list.append(data_index)
    #     assert len(self.query_list) == len(self.index_list)
    #     logger.info(f"final len_query_list:{len(self.query_list)}")
    #     logger.info(f"final len_index_list:{len(self.index_list)}")
    #
    #     # 这里批量一次性整好所有的数据 tokenizer,这里 250w 数据大概需要2个小时,需要200g左右内存，100g 内存程序会崩溃
    #     logger.info(f" 开始批量 tokenizer 所有数据 ... ")
    #     self.all_data_token = self.tokenizer(self.query_list,
    #                                          padding='max_length',
    #                                          truncation=True,
    #                                          max_length=self.max_seq_length,
    #                                          return_tensors='pt')
    #     logger.info(f" 批量 tokenizer 所有数据 完成 ！！！")
    def __init__(self, input_file, tokenizer, max_seq_length):
        self.tokenizer = tokenizer
        self.max_seq_length = max_seq_length
        logger.info('Loading data: {}'.format(input_file))
        with open(input_file, 'r', encoding='utf8') as f:
            data_list = f.readlines()
        logger.info("There are {} data in dataset".format(len(data_list)))

        # 测试：这里仅取 top 100 条数据
        self.raw_data_list = data_list #[:100]

        # 提取对话内容
        self.query_list = []
        self.index_list = []
        for line in self.raw_data_list:
             json_line = json.loads(line)
             # 这个仅仅得到 query 的 embeding, content[0] 是 query
             query = json_line["主要内容"].strip()
             # 这个是原始数据里，单轮对话对应的 data_index
             data_index = json_line["工单编号"]
             data_index = str(data_index)
             self.query_list.append(query)
             self.index_list.append(data_index)

        assert len(self.query_list) == len(self.index_list)
        logger.info(f"final len_query_list:{len(self.query_list)}")
        logger.info(f"final len_index_list:{len(self.index_list)}")

        # 批量 Tokenize 所有数据
        logger.info(f"开始批量 tokenize 所有数据 ...")
        self.all_data_token = self.tokenizer(
            self.query_list,
            padding='max_length',
            truncation=True,
            max_length=self.max_seq_length,
            return_tensors='pt'
        )
        logger.info(f"批量 tokenize 所有数据 完成 ！！！")

    def __getitem__(self, idx):
        # item 包含了 input_ids，attention_mask 等参数,已经转化为 tensor
        item = {key: torch.as_tensor(value[idx]) for key, value in self.all_data_token.items()}
        item['data_index'] = self.index_list[idx]
        return item

    def __len__(self):
        return len(self.query_list)


class TextEmbedder:
    def __init__(self, args):
        self.args = args
        self.device = torch.device("cuda:0")
        self.tokenizer = AutoTokenizer.from_pretrained(self.args.model_name_or_path,
                                                       model_max_length=args.max_length,
                                                       # 这里是推理过程,指定 左侧padding,不然取推理结果有问题
                                                       padding_side="left",
                                                       use_fast=False)
        self.model = AutoModel.from_pretrained(self.args.model_name_or_path).to(self.device)
        logger.info('tokenizer,model 加载完成')
        self.all_dataset = MyTorchDataset(self.args.raw_data_path, self.tokenizer, self.args.max_length)
        self.data_loader = DataLoader(dataset=self.all_dataset, batch_size=self.args.batch_size,
                                      # pin_memory=True,num_workers=1, prefetch_factor=8
                                      )

    def save_data(self, results: List) -> None:
        logger.info(f'需要保存的数据长度: {len(results)}')

        df = pandas.DataFrame(results)
        df.sort_values(by="data_index", inplace=True)
        df.reset_index(drop=True, inplace=True)

        df.to_pickle(self.args.output_path)
        logger.info(f"Saved pickle to {self.args.output_path}")

        '''
        # 这个地方 .csv 会保存失败 
        csv_file_name = "./rl_demo_emb_240624.csv"
        df.to_csv(csv_file_name, index=False)
        logger.info(f"Saved csv to {csv_file_name}")
        '''

        # 指定保存的文件名
        torch_py_bin_name = './12345/embedding/123451wfilter_embedding.pth'
        # 请注意，torch.save()和torch.load()函数不仅可以用于保存和加载张量，还可以用于保存和加载任何Python对象，
        # 包括但不限于字典、集合、自定义类实例等。
        torch.save(results, torch_py_bin_name)
        logger.info(f'List has been saved to {torch_py_bin_name}')

    def encode_samples(self):
        total_samples = len(self.all_dataset)
        logger.info(f"total_samples: {total_samples}")
        total_batches = len(self.data_loader)
        logger.info(f"total_batches: {total_batches}")

        all_embeddings_list = []

        for b_idx, batch in enumerate(tqdm(self.data_loader, total=total_batches)):
            self.model.eval()
            batch_idx = batch["data_index"]
            input_ids = batch['input_ids'].to(self.device)
            attention_mask = batch['attention_mask'].to(self.device)
            output = None
            with torch.no_grad():
                output = self.model(input_ids=input_ids, attention_mask=attention_mask)

            cls_embedding = output.pooler_output.detach().cpu().numpy().tolist()

            # 记录 embedding 和 index
            cur_sample_idx = batch_idx#.tolist()  # 如果 batch_idx 是张量
            cur_sample_dict_list = [
                {"embedding": cls_emb, "data_index": s_id}
                for cls_emb, s_id in zip(cls_embedding, cur_sample_idx)
            ]
            all_embeddings_list.extend(cur_sample_dict_list)

        return all_embeddings_list

    # def encode_samples(self):
    #     total_samples = len(self.all_dataset)
    #     logger.info(f"total_samples:{total_samples}")
    #     total_batches = len(self.data_loader)
    #     logger.info(f"total_batches:{total_batches}")
    #
    #     all_embeddings_list = []
    #
    #     for b_idx, batch in enumerate(tqdm(self.data_loader, total=total_samples // self.args.batch_size)):
    #         self.model.eval()
    #
    #         batch_idx = batch["data_index"]
    #         input_ids = batch['input_ids'].to(self.device)
    #         attention_mask = batch['attention_mask'].to(self.device)
    #         output = None
    #         with torch.no_grad():
    #             output = self.model(input_ids=input_ids, attention_mask=attention_mask)
    #
    #         cls_embedding = output.pooler_output.detach().cpu().numpy().tolist()
    #         print(batch_idx)
    #         # 这里批量记录下 embeding 和 index
    #         cur_sample_idx = batch_idx.tolist()
    #         cur_sample_dict_list = [{"embedding": cls_emb, "data_index": s_id} for cls_emb, s_id in
    #                                 zip(cls_embedding, cur_sample_idx)]
    #         # list extend 多个 list元素 flatten 合并成一个 list
    #         all_embeddings_list.extend(cur_sample_dict_list)
    #
    #     return all_embeddings_list


if __name__ == '__main__':
    torch.cuda.empty_cache()
    # 参数设置
    set_seed()
    # 若干参数设置
    parser = argparse.ArgumentParser()
    # 不要改该参数，系统会自动分配
    parser.add_argument('--device', default='cuda', help='device id (i.e. 0 or 0,1 or cpu)')
    # 开启的进程数(注意不是线程),不用设置该参数，会根据nproc_per_node自动设置
    parser.add_argument('--world_size', default=1, type=int, help='number of distributed processes')
    args = parser.parse_args()

    args.max_length = 512
    args.batch_size = 2048
    args.model_name_or_path = "/model-pvc/suojiayi/bge-base-zh-v1.5/"
    #args.raw_data_path = "/dataset-pvc/suojiayi/new/train_prepare/20250423_020157/tmp_data/instruct_data_COIG_filtered_2504212014.jsonl"
    args.raw_data_path = '/data/zhaochsh01/buquan/12345/tool/output.jsonl'
    args.output_path = "./12345/embedding/123451wffilter_embedding.bin"

    logger.info(f"\nargs:{args}\n")

    text_embeder = TextEmbedder(args)
    all_embeddings_list = text_embeder.encode_samples()

    logger.info(f"len_all_embeddings_list:{len(all_embeddings_list)}")
    logger.info("Finished embedding")

    text_embeder.save_data(all_embeddings_list)
    logger.info(f"Pipeline run complete.")
